--[[
Cursor Cooldown -- Classic Edition :)
Thanks to cncfanatics for the origional code format and ideas to start off on.
Copyright (c) 2007, 2020 by Orionshock

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without
restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.


]]--
--Debug Function
local function D(arg, ...)
	if tostring(arg or ""):find("%%") then
		arg = arg:format(...)
	else
		arg = string.join(", ", tostringall(arg, ...) ):gsub("[=:],", "%1")
	end
	--ChatFrame5:AddMessage("CursorCooldown: "..arg)
end

--Addon Decleration
local addonName, addon = ...
if _G[addonName] ~= addon then
	_G[addonName] = addon
end

--UpVariables
local GetSpellInfo = GetSpellInfo
local GetSpellCooldown = GetSpellCooldown

--Hard Coded Variables
addon.useSpellTexture = false

--Spell Checking & Updating::
local function foundSpell()
	return addon.spell
end
local function GSC_GSI(id)
	--if we pass the spell id into GetSpellCooldown - it always returns
	--but if we pass the localized name in, it won't
	return GetSpellCooldown( ( GetSpellInfo(id) ) )
end

local function checkSpell()
	if addon.spell then
		return addon.spell
	else
		local spell = nil
		if GSC_GSI(1126) then			--Druid	Mark of the Wild	~done
			spell = GetSpellInfo(1126)
		elseif GSC_GSI(1494) then			--Hunter	Track Beasts	~done	acts weird with AutoShot but works fine..
			spell = GetSpellInfo(1494)
		elseif GSC_GSI(168) then			--Mage	Frost Armor	~done
			spell = GetSpellInfo(168)
		elseif GSC_GSI(465) then			--Pally	Devotion Aura	~done
			spell = GetSpellInfo(465)
		elseif GSC_GSI(1243) then			--Priest	Power Word: Fortitude	~done
			spell = GetSpellInfo(1243)
		elseif GSC_GSI(1752) then			--Rouge	Sinister Strike	~done
			spell = GetSpellInfo(1752)
		elseif GSC_GSI(8017) then			--shammy	Rockbiter Weapon	~done
			spell = GetSpellInfo(8017)
		elseif GSC_GSI(687) then			--Warlock	Demon Skin	~done
			spell = GetSpellInfo(687)
		elseif GSC_GSI(6673) then			--Warrior	Battle Shout	~done; Nothing working at l1
			spell = GetSpellInfo(6673)
		end
		
		if spell then 
			addon.spell = spell
			checkSpell = foundSpell
			return spell
		else
			return nil
		end
	end
	return nil
end

--THE FRAME--
--Create BaseFrame
local frame = CreateFrame("Frame", nil, UIParent)
addon.frame = frame
frame:SetFrameStrata("TOOLTIP")
frame:SetFrameLevel(1)
frame:SetWidth(20)
frame:SetHeight(20)
frame:Show()

--Texture on the Frame
local texture = frame:CreateTexture(nil, frame:GetFrameLevel()+1)
addon.texture = texture
texture:SetAllPoints(frame)
texture:Show()

--Cooldown Frame that sits on Texture
local cdF = CreateFrame("Cooldown", nil, frame, "CooldownFrameTemplate");
addon.cdF = cdF
cdF:SetFrameStrata("TOOLTIP")
cdF:SetFrameLevel(frame:GetFrameLevel()+2)
cdF:RegisterEvent("UNIT_SPELLCAST_START")
cdF:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
cdF:SetAllPoints(frame)
cdF:Show()

--Frame Scripts
local start, duration, enable, x, y, Scale, sX, sY
frame:SetScript("OnUpdate", function(self, update)
	x, y = GetCursorPosition();
	x = x + 15
	y = y - 40
	Scale = UIParent:GetScale();
	sX = x / Scale;
	sY = y / Scale;
	self:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", sX, sY);
	
	if not checkSpell() then return end

	start, duration, enable = GetSpellCooldown(addon.spell)

	if ( start > 0 and duration > 0 and enable > 0) then
		cdF:SetCooldown(start, duration);
		cdF:Show()
		frame:Show()
	else
		cdF:SetCooldown(0,0)
		if not (UnitCastingInfo("player") or UnitChannelInfo("player")) then
			self:Hide()
		end
	end
end);

local solidTexture = "Interface\\Icons\\Temp"
local function setTexture(spellTexture)
	if addon.useSpellTexture then
		texture:SetTexture(spellTexture)
		frame:Show()
	else
		texture:SetTexture(solidTexture)
		frame:Show()
	end
end

local GetCastInfo
	--This is because args changed in every version, thank you blizzard!
do
	local isRetail = WOW_PROJECT_ID == WOW_PROJECT_MAINLINE
	local isTBC = WOW_PROJECT_ID == WOW_PROJECT_BURNING_CRUSADE_CLASSIC
	local isClassic = WOW_PROJECT_ID == WOW_PROJECT_CLASSIC
	
	if isRetail then
		function GetCastInfo(id)
			local spellName, _, spellTexture, _, _, _, _, _, cSpellId = UnitCastingInfo("player")	--Casted Spell - args: 1,3,9
			if not spellName then
				spellName, _, spellTexture, _, _, _, _, cSpellId = UnitChannelInfo("player")		--Channeled Spell - args: 1,3,8
			end
			if not spellName then 
				spellName, _, spellTexture, _, _, _, cSpellId = GetSpellInfo(id)
			end
			return spellName, spellTexture, cSpellId
		end
	elseif isTBC then
		function GetCastInfo(id)
			local spellName, _, spellTexture, _, _, _, _, cSpellId = UnitCastingInfo("player")		--Casted Spell - args: 1,3,8
			if not spellName then
				spellName, _, spellTexture, _, _, _, cSpellId = UnitChannelInfo("player")			--Channeled Spell - args: 1,3,7
			end
			if not spellName then 
				spellName, _, spellTexture, _, _, _, cSpellId = GetSpellInfo(id)
			end
			return spellName, spellTexture, cSpellId
		end
	elseif isClassic then
		function GetCastInfo(id)
			local spellName, _, spellTexture, _, _, _, _, _, cSpellId = CastingInfo()				--Casted Spell - args: 1,3,9
			if not spellName then
				spellName, _, spellTexture, _, _, _, _, cSpellId = ChannelInfo()					--Channeled Spell - args: 1,3,8
			end
			if not spellName then 
				spellName, _, spellTexture, _, _, _, cSpellId = GetSpellInfo(id)
			end
			return spellName, spellTexture, cSpellId
		end
	end

end

cdF:SetScript("OnEvent", function(self, event, ...)
	if not checkSpell() then return end
	local unitTarget, _, eSpellId = ...
	if unitTarget == "player" then	--Only Care about unitTarget being the player
		local spellName, spellTexture, cSpellId = GetCastInfo(eSpellId)
		if cSpellId and (cSpellId == eSpellId) then
			setTexture(spellTexture)
		end
	end
end)


SLASH_CURSORCOOLDOWN1 = "/ccd"
SlashCmdList["CURSORCOOLDOWN"] = function(msg, editBox, ...)
	print(msg, editBox, ...)
	msg = (msg or ""):lower():trim()
	if msg == "" then
		print("CursorCooldown: Use '/ccd texture' or '/ccd solid' to switch")
		return
	elseif msg == "texture" then
		addon.useSpellTexture = true
		print("CursorCooldown: Now using spell icon on cursor")
	elseif msg == "solid" then
		addon.useSpellTexture = false
		print("CursorCooldown: Now using solid icon on cursor")
	else
		addon.useSpellTexture = true
	end
end


---Addon Frame Event Handler

frame:SetScript("OnEvent", function(self, event, ...)
	if event == "VARIABLES_LOADED" then
		CooldownCountSpell_SV = CooldownCountSpell_SV or { useSpellTexture = true }
		addon.useSpellTexture = CooldownCountSpell_SV.useSpellTexture
	elseif event == "PLAYER_LOGOUT" then
		CooldownCountSpell_SV.useSpellTexture = addon.useSpellTexture
	end
end)
frame:RegisterEvent("VARIABLES_LOADED")
frame:RegisterEvent("PLAYER_LOGOUT")